% estse.m: Maximizes the (minimizes the negative) log likelihood
%            function for the model of money's role.
%
%          When maximizing the log likelihood function, the parameters
%            are transformed to satisfy theoretical restrictions.
%            The log likelihood function with transformed parameters
%            is contained in llfn.m.
%
%          When calculating standard errors, the original
%            parameters are used.  The log likelihood function without
%            transformed parameters is contained in llfnse.m.
%
%          The untransformed parameter estimates are reported as
%            the vector tstar.  The standard errors are reported as
%            the vector sevec.
%
%          The program also computes standard errors for the values
%            of omega1, gamma3, and beta that are implied by the
%            estimates of the other parameters.  The implied parameter
%            values are reported as the vector betstar and the standard
%            errors are reported as the vector betse.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% load and detrend M2 data

%  global yt mt pt rt scalinv

%  load ympr2.dat;

%  yt = ympr2(84:170,1);
%  mt = ympr2(84:170,2);
%  pt = ympr2(84:170,3);
%  rt = ympr2(84:170,4);

%  bigt = length(yt);

%  trend = 1:bigt;

%  yt = yt - 0.005074*trend';
%  mt = mt - 0.002644*trend';

% load and detrend M1 data

  global yt mt pt rt scalinv

  load ympr1.dat;

  yt = ympr1(84:170,1);
  mt = ympr1(84:170,2);
  pt = ympr1(84:170,3);
  rt = ympr1(84:170,4);

  bigt = length(yt);

  trend = 1:bigt;

  yt = yt - 0.005074*trend';
  mt = mt - 0.003323*trend';

% set starting values for M2, omega2 estimated

%  omega1tr = 1;
%  omega2tr = 0.1;
%  gamma2tr = 1;
%  psitr = 0.1;
%  rhortr = 0.75;
%  rhoytr = 0.05;
%  rhoptr = 0.35;
%  rhomtr = 0.00000001;
%  lnysstr = 8.8619;
%  lnmsstr = 9.7248;
%  lnpsstr = 0.0082;
%  lnrsstr = 0.0169;
%  rhoatr = 0.3;
%  rhoetr = 0.3;
%  rhoztr = 0.3;
%  siga = 0.0293;
%  sige = 0.0101;
%  sigz = 0.0083;
%  sigr = 0.0025;

%  bigtheto = [ omega2tr gamma2tr ...
%               rhortr rhoytr rhoptr ...
%               lnysstr lnmsstr lnpsstr lnrsstr ...
%               rhoatr rhoetr rhoztr ...
%               siga sige sigz sigr ]';

%  scalvec = [ 1 1 1 1 1 0.1 0.1 10 10 1 1 1 10 10 10 10 ]'; 

% set starting values for M2, omega2 fixed

%  omega1tr = 1;
%  omega2tr = 0.25;
%  gamma2tr = 1;
%  psitr = 0.1;
%  rhortr = 0.75;
%  rhoytr = 0.05;
%  rhoptr = 0.35;
%  rhomtr = 0.00000001;
%  lnysstr = 8.8619;
%  lnmsstr = 9.7248;
%  lnpsstr = 0.0082;
%  lnrsstr = 0.0169;
%  rhoatr = 0.3;
%  rhoetr = 0.3;
%  rhoztr = 0.3;
%  siga = 0.0293;
%  sige = 0.0100;
%  sigz = 0.0083;
%  sigr = 0.0025;

%  bigtheto = [ gamma2tr ...
%               rhortr rhoytr rhoptr ...
%               lnysstr lnmsstr lnpsstr lnrsstr ...
%               rhoatr rhoetr rhoztr ...
%               siga sige sigz sigr ]';

%  scalvec = [ 1 1 1 1 0.1 0.1 10 10 1 1 1 10 10 10 10 ]'; 

% set starting values for M1, omega2 estimated

  omega1tr = 1;
  omega2tr = 0.1;
  gamma2tr = 1;
  psitr = 0.1;
  rhortr = 0.75;
  rhoytr = 0.05;
  rhoptr = 0.35;
  rhomtr = 0.00000001;
  lnysstr = 8.8619;
  lnmsstr = 8.3579;
  lnpsstr = 0.0082;
  lnrsstr = 0.0169;
  rhoatr = 0.3;
  rhoetr = 0.3;
  rhoztr = 0.3;
  siga = 0.0292;
  sige = 0.0173;
  sigz = 0.0082;
  sigr = 0.0025;

  bigtheto = [ omega2tr gamma2tr ...
               rhortr rhoytr rhoptr ...
               lnysstr lnmsstr lnpsstr lnrsstr ...
               rhoatr rhoetr rhoztr ...
               siga sige sigz sigr ]';

  scalvec = [ 10 1 1 1 1 0.1 0.1 10 10 1 1 1 10 10 10 10 ]'; 

% maximize likelihood

  options(1) = 1;
  options(14) = 10000;

  thetstar = fminu('llfn',bigtheto,options);

% calculate standard errors when omega2 is estimated

  thetstar = real(thetstar);

  omega1 = abs(omega1tr);
  omega2 = abs(thetstar(1));
  gamma2 = abs(thetstar(2));
  psi = abs(psitr);
  rhor = abs(thetstar(3));
  rhoy = abs(thetstar(4));
  rhop = abs(thetstar(5));
  rhom = abs(rhomtr);
  lnyss = abs(thetstar(6));
  lnmss = abs(thetstar(7));
  lnpss = abs(thetstar(8));
  lnrss = abs(thetstar(9));
  rhoa = ((10*thetstar(10))^2)/(1+(10*thetstar(10))^2);
  rhoe = ((10*thetstar(11))^2)/(1+(10*thetstar(11))^2);
  rhoz = ((10*thetstar(12))^2)/(1+(10*thetstar(12))^2);
  siga = abs(thetstar(13));
  sige = abs(thetstar(14));
  sigz = abs(thetstar(15));
  sigr = abs(thetstar(16));

  tstar = [ omega2 gamma2 ...
            rhor rhoy rhop ...
            lnyss lnmss lnpss lnrss ...
            rhoa rhoe rhoz ...
            siga sige sigz sigr ]';

  scalinv = inv(diag(scalvec));

  tstars = diag(scalvec)*tstar;

  fstar = llfnse(tstars);

  eee = 1e-6;

  epsmat = eee*eye(16);

  hessvec = zeros(16,1);

  for i = 1:16

    hessvec(i) = llfnse(tstars+epsmat(:,i));

  end

  hessmatl = zeros(16,16);

  for i = 1:16

    for j = 1:i

      hessmatl(i,j) = (llfnse(tstars+epsmat(:,i)+epsmat(:,j)) ...
                        -hessvec(i)-hessvec(j)+fstar)/(eee^2);

    end

  end

  hessmatu = hessmatl' - diag(diag(hessmatl));

  hessmatf = hessmatl + hessmatu;

  bighx = scalinv*inv(hessmatf)*scalinv';

  sevec = sqrt(diag(bighx));

  yss = exp(lnyss);
  mss = exp(lnmss);
  pss = exp(lnpss);
  rss = exp(lnrss);

  gamma1 = (rss-1+yss*rss*omega2/mss)*(gamma2/omega1);
  gamma3 = 1 - (rss-1)*gamma2;
  bet = pss/rss;

  betstar = betfn(tstar);

  eee = 1e-6;

  epsmat = eee*eye(16);

  betgrad = zeros(3,16);

  for i = 1:16

    betgrad(:,i) = (betfn(tstar+epsmat(:,i))-betstar)/eee;

  end

  betse = sqrt(diag(betgrad*bighx*betgrad'));

% calculate standard errors when omega2 is fixed

%  thetstar = real(thetstar);

%  omega1 = abs(omega1tr);
%  omega2 = abs(omega2tr);
%  gamma2 = abs(thetstar(1));
%  psi = abs(psitr);
%  rhor = abs(thetstar(2));
%  rhoy = abs(thetstar(3));
%  rhop = abs(thetstar(4));
%  rhom = abs(rhomtr);
%  lnyss = abs(thetstar(5));
%  lnmss = abs(thetstar(6));
%  lnpss = abs(thetstar(7));
%  lnrss = abs(thetstar(8));
%  rhoa = ((10*thetstar(9))^2)/(1+(10*thetstar(9))^2);
%  rhoe = ((10*thetstar(10))^2)/(1+(10*thetstar(10))^2);
%  rhoz = ((10*thetstar(11))^2)/(1+(10*thetstar(11))^2);
%  siga = abs(thetstar(12));
%  sige = abs(thetstar(13));
%  sigz = abs(thetstar(14));
%  sigr = abs(thetstar(15));

%  tstar = [ gamma2 ...
%            rhor rhoy rhop ...
%            lnyss lnmss lnpss lnrss ...
%            rhoa rhoe rhoz ...
%            siga sige sigz sigr ]';

%  scalinv = inv(diag(scalvec));

%  tstars = diag(scalvec)*tstar;

%  fstar = llfnse(tstars);

%  eee = 1e-6;

%  epsmat = eee*eye(15);

%  hessvec = zeros(15,1);

%  for i = 1:15

%    hessvec(i) = llfnse(tstars+epsmat(:,i));

%  end

%  hessmatl = zeros(15,15);

%  for i = 1:15

%    for j = 1:i

%      hessmatl(i,j) = (llfnse(tstars+epsmat(:,i)+epsmat(:,j)) ...
%                        -hessvec(i)-hessvec(j)+fstar)/(eee^2);

%    end

%  end

%  hessmatu = hessmatl' - diag(diag(hessmatl));

%  hessmatf = hessmatl + hessmatu;

%  bighx = scalinv*inv(hessmatf)*scalinv';

%  sevec = sqrt(diag(bighx));

%  yss = exp(lnyss);
%  mss = exp(lnmss);
%  pss = exp(lnpss);
%  rss = exp(lnrss);

%  gamma1 = (rss-1+yss*rss*omega2/mss)*(gamma2/omega1);
%  gamma3 = 1 - (rss-1)*gamma2;
%  bet = pss/rss;

%  betstar = betfn(tstar);

%  eee = 1e-6;

%  epsmat = eee*eye(15);

%  betgrad = zeros(3,15);

%  for i = 1:15

%    betgrad(:,i) = (betfn(tstar+epsmat(:,i))-betstar)/eee;

%  end

%  betse = sqrt(diag(betgrad*bighx*betgrad'));