% vardec.m: Performs variance decompositions for the model of the 
%             money's role.
%
%           Produces 
%
%             varvec = [ varvecy
%                        varvecm
%                        varvecp
%                        varvecr ]
%
%             where each varvecx is a 7x1 vector of k-step ahead
%             forecast error variances for k=1,4,8,12,20,40,inf.
%
%           Also produces
%
%             vardecm = [ vardecmy
%                         vardecmm
%                         vardecmp
%                         vardecmr ]
%
%             where each vardecmx is a 7x4 matrix that decomposes
%             the variances in varvecx into percentages due to each
%             of the four shocks: IS, LM, supply, and policy.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% set parameter values

%  omega1 = 1;
%  omega2 = 0.0000;
%  gamma2 = 0.7220;
%  psi = 0.1;
%  rhor = 0.5481;
%  rhoy = 0.0000;
%  rhop = 0.5680;
%  rhom = 0.00000001;
%  lnyss = 8.8731;
%  lnmss = 9.7381;
%  lnpss = 0.0102;
%  lnrss = 0.0189;
%  rhoa = 0.9579;
%  rhoe = 0.9853;
%  rhoz = 0.9903;
%  siga = 0.0188;
%  sige = 0.0088;
%  sigz = 0.0098;
%  sigr = 0.0025;

  omega1 = 1;
  omega2 = 0.25;
  gamma2 = 0.1251;
  psi = 0.1;
  rhor = 0.5481;
  rhoy = 0.0000;
  rhop = 0.5680;
  rhom = 0.00000001;
  lnyss = 8.8747;
  lnmss = 9.7383;
  lnpss = 0.0100;
  lnrss = 0.0186;
  rhoa = 0.9575;
  rhoe = 0.9867;
  rhoz = 0.9904;
  siga = 0.0187;
  sige = 0.0088;
  sigz = 0.0098;
  sigr = 0.0025;

  yss = exp(lnyss);
  mss = exp(lnmss);
  pss = exp(lnpss);
  rss = exp(lnrss);

  gamma1 = (rss-1+yss*rss*omega2/mss)*(gamma2/omega1);
  gamma3 = 1 - (rss-1)*gamma2;

% form matrices A, B, and C

  biga = zeros(3,3);
  bigb = zeros(3,7);
  bigc = zeros(3,4);

  biga(1,1) = 1;
  biga(1,2) = gamma2;
  bigb(1,6) = gamma1;
  bigc(1,2) = gamma3;

  biga(2,1) = 1;
  biga(2,3) = -1;
  bigb(2,2) = 1;
  bigb(2,7) = -1;

  biga(3,2) = 1;
  bigb(3,1) = rhoy;
  bigb(3,3) = rhop;
  bigb(3,4) = rhor;
  bigb(3,5) = rhom;
  bigc(3,4) = 1;

% form matrices D, F, G, H, and J

  bigd = zeros(7,7);
  bigf = zeros(7,3);
  bigg = zeros(7,7);
  bigh = zeros(7,3);
  bigj = zeros(7,4);

  bigd(1,6) = 1;
  bigd(1,7) = omega1;
  bigf(1,1) = -omega2;
  bigg(1,6) = 1;
  bigh(1,1) = -omega2;
  bigh(1,2) = omega1;
  bigj(1,1) = -omega1*(1-rhoa);
  bigj(1,2) = omega2*(1-rhoe);

  bigd(2,7) = pss/rss;
  bigg(2,6) = -psi/omega1;
  bigg(2,7) = 1;
  bigh(2,1) = psi*(omega2/omega1);
  bigj(2,2) = -psi*(omega2/omega1);
  bigj(2,3) = psi;

  bigd(3,1) = 1;
  bigg(3,6) = 1;

  bigd(4,2) = 1;
  bigh(4,1) = 1;

  bigd(5,3) = 1;
  bigg(5,7) = 1;

  bigd(6,4) = 1;
  bigh(6,2) = 1;

  bigd(7,5) = 1;
  bigh(7,3) = 1;

% form matrix P

  bigp = zeros(4,4);

  bigp(1,1) = rhoa;
  bigp(2,2) = rhoe;
  bigp(3,3) = rhoz;

% form matrices K and L

  bigkl1 = inv(bigd+bigf*inv(biga)*bigb);
  bigkl2 = bigg + bigh*inv(biga)*bigb;
  bigkl3 = bigj + bigh*inv(biga)*bigc - bigf*inv(biga)*bigc*bigp;

  bigk = bigkl1*bigkl2;

  bigl = bigkl1*bigkl3;

  bigl1 = bigl(1:5,:);
  bigl2 = bigl(6:7,:);

% form matrices M and N

  [kvec,keig] = eig(bigk);

  keig2 = diag(keig);

  [keig3,kord] = sort(abs(keig2));

  if keig3(5) > 1
    'error - no solution'
  end

  if keig3(6) < 1
    'error - multiple solutions'
  end

  keig4 = keig2(kord);

  bign = diag(keig4);

  bign1 = bign(1:5,1:5);
  bign2 = bign(6:7,6:7);

  bigminv = kvec(:,kord);

  bigm = inv(bigminv);

  bigm11 = bigm(1:5,1:5);
  bigm12 = bigm(1:5,6:7);
  bigm21 = bigm(6:7,1:5);
  bigm22 = bigm(6:7,6:7);

% form matrices Q and R

  bigq1 = bigm11*bigl1 + bigm12*bigl2;
  bigq2 = bigm21*bigl1 + bigm22*bigl2;

  vecr = inv(eye(8)-kron(bigp,inv(bign2)))*bigq2(:);

  bigr = reshape(vecr,2,4);

% form matrices S

  bigs1 = -inv(bigm22)*bigm21;

  bigs2 = -inv(bigm22)*inv(bign2)*bigr;

  bigs34 = bigm11 + bigm12*bigs1;

  bigs3 = inv(bigs34)*bign1*bigs34;

  bigs4 = inv(bigs34)*(bigq1+bign1*bigm12*bigs2-bigm12*bigs2*bigp);

  bigs5a = [ eye(5) ; bigs1 ]; 

  bigs5 = inv(biga)*bigb*bigs5a;

  bigs6a = [ zeros(5,4) ; bigs2 ];

  bigs6 = inv(biga)*bigb*bigs6a + inv(biga)*bigc;

% form matrices PI, W, and U

  bigpi = [ bigs3 bigs4 ; zeros(4,5) bigp ];

  bigw = [ zeros(5,4) ; eye(4) ];

  bigu = [ bigs5 bigs6 ; bigs1 bigs2 ];

  bigpi = real(bigpi);

  bigu = real(bigu);

% form matrices AX, BX, CX, VX, and BVBX

  bigax = bigpi;

  bigbx = bigw;

  bigcx = [ bigu(4,:) ; bigu(1,:) ; bigu(5,:) ; bigu(2,:) ];

  bigvx = diag([siga^2 sige^2 sigz^2 sigr^2]);

  bigbvbx = bigbx*bigvx*bigbx';

% set up output matrices

  varvec = zeros(28,1);

  vardecm = zeros(28,4);

% calculate k-step ahead forecast error variances

  bigakx = eye(9);

  bigsigsk = zeros(9,9);

  for k = 1:40

    bigsigsk = bigsigsk + bigakx*bigbvbx*bigakx';

    bigsigdk = bigcx*bigsigsk*bigcx';

    if k == 1
      varvec(1) = bigsigdk(1,1);
      varvec(8) = bigsigdk(2,2);
      varvec(15) = bigsigdk(3,3);
      varvec(22) = bigsigdk(4,4);
    end

    if k == 4
      varvec(2) = bigsigdk(1,1);
      varvec(9) = bigsigdk(2,2);
      varvec(16) = bigsigdk(3,3);
      varvec(23) = bigsigdk(4,4);
    end

    if k == 8
      varvec(3) = bigsigdk(1,1);
      varvec(10) = bigsigdk(2,2);
      varvec(17) = bigsigdk(3,3);
      varvec(24) = bigsigdk(4,4);
    end

    if k == 12
      varvec(4) = bigsigdk(1,1);
      varvec(11) = bigsigdk(2,2);
      varvec(18) = bigsigdk(3,3);
      varvec(25) = bigsigdk(4,4);
    end

    if k == 20
      varvec(5) = bigsigdk(1,1);
      varvec(12) = bigsigdk(2,2);
      varvec(19) = bigsigdk(3,3);
      varvec(26) = bigsigdk(4,4);
    end

    if k == 40
      varvec(6) = bigsigdk(1,1);
      varvec(13) = bigsigdk(2,2);
      varvec(20) = bigsigdk(3,3);
      varvec(27) = bigsigdk(4,4);
    end

    bigakx = bigakx*bigax;

  end    

  bigsigs1 = inv(eye(81)-kron(bigax,bigax))*bigbvbx(:);

  bigsigs = reshape(bigsigs1,9,9);

  bigsigd = bigcx*bigsigs*bigcx';

  varvec(7) = bigsigd(1,1);
  varvec(14) = bigsigd(2,2);
  varvec(21) = bigsigd(3,3);
  varvec(28) = bigsigd(4,4);

% calculate fractions due to IS shocks

  bigvx = diag([siga^2 0 0 0]);

  bigbvbx = bigbx*bigvx*bigbx';

  bigakx = eye(9);

  bigsigsk = zeros(9,9);

  for k = 1:40

    bigsigsk = bigsigsk + bigakx*bigbvbx*bigakx';

    bigsigdk = bigcx*bigsigsk*bigcx';

    if k == 1
      vardecm(1,1) = bigsigdk(1,1)/varvec(1);
      vardecm(8,1) = bigsigdk(2,2)/varvec(8);
      vardecm(15,1) = bigsigdk(3,3)/varvec(15);
      vardecm(22,1) = bigsigdk(4,4)/varvec(22);
    end

    if k == 4
      vardecm(2,1) = bigsigdk(1,1)/varvec(2);
      vardecm(9,1) = bigsigdk(2,2)/varvec(9);
      vardecm(16,1) = bigsigdk(3,3)/varvec(16);
      vardecm(23,1) = bigsigdk(4,4)/varvec(23);
    end

    if k == 8
      vardecm(3,1) = bigsigdk(1,1)/varvec(3);
      vardecm(10,1) = bigsigdk(2,2)/varvec(10);
      vardecm(17,1) = bigsigdk(3,3)/varvec(17);
      vardecm(24,1) = bigsigdk(4,4)/varvec(24);
    end

   if k == 12
      vardecm(4,1) = bigsigdk(1,1)/varvec(4);
      vardecm(11,1) = bigsigdk(2,2)/varvec(11);
      vardecm(18,1) = bigsigdk(3,3)/varvec(18);
      vardecm(25,1) = bigsigdk(4,4)/varvec(25);
    end

    if k == 20
      vardecm(5,1) = bigsigdk(1,1)/varvec(5);
      vardecm(12,1) = bigsigdk(2,2)/varvec(12);
      vardecm(19,1) = bigsigdk(3,3)/varvec(19);
      vardecm(26,1) = bigsigdk(4,4)/varvec(26);
    end

    if k == 40
      vardecm(6,1) = bigsigdk(1,1)/varvec(6);
      vardecm(13,1) = bigsigdk(2,2)/varvec(13);
      vardecm(20,1) = bigsigdk(3,3)/varvec(20);
      vardecm(27,1) = bigsigdk(4,4)/varvec(27);
    end

    bigakx = bigakx*bigax;

  end    

  bigsigs1 = inv(eye(81)-kron(bigax,bigax))*bigbvbx(:);

  bigsigs = reshape(bigsigs1,9,9);

  bigsigd = bigcx*bigsigs*bigcx';

  vardecm(7,1) = bigsigd(1,1)/varvec(7);
  vardecm(14,1) = bigsigd(2,2)/varvec(14);
  vardecm(21,1) = bigsigd(3,3)/varvec(21);
  vardecm(28,1) = bigsigd(4,4)/varvec(28);

% calculate fractions due to LM shocks

  bigvx = diag([0 sige^2 0 0]);

  bigbvbx = bigbx*bigvx*bigbx';

  bigakx = eye(9);

  bigsigsk = zeros(9,9);

  for k = 1:40

    bigsigsk = bigsigsk + bigakx*bigbvbx*bigakx';

    bigsigdk = bigcx*bigsigsk*bigcx';

    if k == 1
      vardecm(1,2) = bigsigdk(1,1)/varvec(1);
      vardecm(8,2) = bigsigdk(2,2)/varvec(8);
      vardecm(15,2) = bigsigdk(3,3)/varvec(15);
      vardecm(22,2) = bigsigdk(4,4)/varvec(22);
    end

    if k == 4
      vardecm(2,2) = bigsigdk(1,1)/varvec(2);
      vardecm(9,2) = bigsigdk(2,2)/varvec(9);
      vardecm(16,2) = bigsigdk(3,3)/varvec(16);
      vardecm(23,2) = bigsigdk(4,4)/varvec(23);
    end

    if k == 8
      vardecm(3,2) = bigsigdk(1,1)/varvec(3);
      vardecm(10,2) = bigsigdk(2,2)/varvec(10);
      vardecm(17,2) = bigsigdk(3,3)/varvec(17);
      vardecm(24,2) = bigsigdk(4,4)/varvec(24);
    end

   if k == 12
      vardecm(4,2) = bigsigdk(1,1)/varvec(4);
      vardecm(11,2) = bigsigdk(2,2)/varvec(11);
      vardecm(18,2) = bigsigdk(3,3)/varvec(18);
      vardecm(25,2) = bigsigdk(4,4)/varvec(25);
    end

    if k == 20
      vardecm(5,2) = bigsigdk(1,1)/varvec(5);
      vardecm(12,2) = bigsigdk(2,2)/varvec(12);
      vardecm(19,2) = bigsigdk(3,3)/varvec(19);
      vardecm(26,2) = bigsigdk(4,4)/varvec(26);
    end

    if k == 40
      vardecm(6,2) = bigsigdk(1,1)/varvec(6);
      vardecm(13,2) = bigsigdk(2,2)/varvec(13);
      vardecm(20,2) = bigsigdk(3,3)/varvec(20);
      vardecm(27,2) = bigsigdk(4,4)/varvec(27);
    end

    bigakx = bigakx*bigax;

  end    

  bigsigs1 = inv(eye(81)-kron(bigax,bigax))*bigbvbx(:);

  bigsigs = reshape(bigsigs1,9,9);

  bigsigd = bigcx*bigsigs*bigcx';

  vardecm(7,2) = bigsigd(1,1)/varvec(7);
  vardecm(14,2) = bigsigd(2,2)/varvec(14);
  vardecm(21,2) = bigsigd(3,3)/varvec(21);
  vardecm(28,2) = bigsigd(4,4)/varvec(28);

% calculate fractions due to supply shocks

  bigvx = diag([0 0 sigz^2 0]);

  bigbvbx = bigbx*bigvx*bigbx';

  bigakx = eye(9);

  bigsigsk = zeros(9,9);

  for k = 1:40

    bigsigsk = bigsigsk + bigakx*bigbvbx*bigakx';

    bigsigdk = bigcx*bigsigsk*bigcx';

    if k == 1
      vardecm(1,3) = bigsigdk(1,1)/varvec(1);
      vardecm(8,3) = bigsigdk(2,2)/varvec(8);
      vardecm(15,3) = bigsigdk(3,3)/varvec(15);
      vardecm(22,3) = bigsigdk(4,4)/varvec(22);
    end

    if k == 4
      vardecm(2,3) = bigsigdk(1,1)/varvec(2);
      vardecm(9,3) = bigsigdk(2,2)/varvec(9);
      vardecm(16,3) = bigsigdk(3,3)/varvec(16);
      vardecm(23,3) = bigsigdk(4,4)/varvec(23);
    end

    if k == 8
      vardecm(3,3) = bigsigdk(1,1)/varvec(3);
      vardecm(10,3) = bigsigdk(2,2)/varvec(10);
      vardecm(17,3) = bigsigdk(3,3)/varvec(17);
      vardecm(24,3) = bigsigdk(4,4)/varvec(24);
    end

   if k == 12
      vardecm(4,3) = bigsigdk(1,1)/varvec(4);
      vardecm(11,3) = bigsigdk(2,2)/varvec(11);
      vardecm(18,3) = bigsigdk(3,3)/varvec(18);
      vardecm(25,3) = bigsigdk(4,4)/varvec(25);
    end

    if k == 20
      vardecm(5,3) = bigsigdk(1,1)/varvec(5);
      vardecm(12,3) = bigsigdk(2,2)/varvec(12);
      vardecm(19,3) = bigsigdk(3,3)/varvec(19);
      vardecm(26,3) = bigsigdk(4,4)/varvec(26);
    end

    if k == 40
      vardecm(6,3) = bigsigdk(1,1)/varvec(6);
      vardecm(13,3) = bigsigdk(2,2)/varvec(13);
      vardecm(20,3) = bigsigdk(3,3)/varvec(20);
      vardecm(27,3) = bigsigdk(4,4)/varvec(27);
    end

    bigakx = bigakx*bigax;

  end    

  bigsigs1 = inv(eye(81)-kron(bigax,bigax))*bigbvbx(:);

  bigsigs = reshape(bigsigs1,9,9);

  bigsigd = bigcx*bigsigs*bigcx';

  vardecm(7,3) = bigsigd(1,1)/varvec(7);
  vardecm(14,3) = bigsigd(2,2)/varvec(14);
  vardecm(21,3) = bigsigd(3,3)/varvec(21);
  vardecm(28,3) = bigsigd(4,4)/varvec(28);

% calculate fractions due to policy shocks

  bigvx = diag([0 0 0 sigr^2]);

  bigbvbx = bigbx*bigvx*bigbx';

  bigakx = eye(9);

  bigsigsk = zeros(9,9);

  for k = 1:40

    bigsigsk = bigsigsk + bigakx*bigbvbx*bigakx';

    bigsigdk = bigcx*bigsigsk*bigcx';

    if k == 1
      vardecm(1,4) = bigsigdk(1,1)/varvec(1);
      vardecm(8,4) = bigsigdk(2,2)/varvec(8);
      vardecm(15,4) = bigsigdk(3,3)/varvec(15);
      vardecm(22,4) = bigsigdk(4,4)/varvec(22);
    end

    if k == 4
      vardecm(2,4) = bigsigdk(1,1)/varvec(2);
      vardecm(9,4) = bigsigdk(2,2)/varvec(9);
      vardecm(16,4) = bigsigdk(3,3)/varvec(16);
      vardecm(23,4) = bigsigdk(4,4)/varvec(23);
    end

    if k == 8
      vardecm(3,4) = bigsigdk(1,1)/varvec(3);
      vardecm(10,4) = bigsigdk(2,2)/varvec(10);
      vardecm(17,4) = bigsigdk(3,3)/varvec(17);
      vardecm(24,4) = bigsigdk(4,4)/varvec(24);
    end

   if k == 12
      vardecm(4,4) = bigsigdk(1,1)/varvec(4);
      vardecm(11,4) = bigsigdk(2,2)/varvec(11);
      vardecm(18,4) = bigsigdk(3,3)/varvec(18);
      vardecm(25,4) = bigsigdk(4,4)/varvec(25);
    end

    if k == 20
      vardecm(5,4) = bigsigdk(1,1)/varvec(5);
      vardecm(12,4) = bigsigdk(2,2)/varvec(12);
      vardecm(19,4) = bigsigdk(3,3)/varvec(19);
      vardecm(26,4) = bigsigdk(4,4)/varvec(26);
    end

    if k == 40
      vardecm(6,4) = bigsigdk(1,1)/varvec(6);
      vardecm(13,4) = bigsigdk(2,2)/varvec(13);
      vardecm(20,4) = bigsigdk(3,3)/varvec(20);
      vardecm(27,4) = bigsigdk(4,4)/varvec(27);
    end

    bigakx = bigakx*bigax;

  end    

  bigsigs1 = inv(eye(81)-kron(bigax,bigax))*bigbvbx(:);

  bigsigs = reshape(bigsigs1,9,9);

  bigsigd = bigcx*bigsigs*bigcx';

  vardecm(7,4) = bigsigd(1,1)/varvec(7);
  vardecm(14,4) = bigsigd(2,2)/varvec(14);
  vardecm(21,4) = bigsigd(3,3)/varvec(21);
  vardecm(28,4) = bigsigd(4,4)/varvec(28);

% convert to percentages

  varvec = 100*varvec;

  vardecm = 100*vardecm;